/*!
 * ol-geocoder - v3.3.0
 * A geocoder extension for OpenLayers.
 * https://github.com/jonataswalker/ol-geocoder
 * Built: Thu Jul 04 2019 07:15:42 GMT-0300 (Brasilia Standard Time)
 */

(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('ol/control/Control'), require('ol/style/Style'), require('ol/style/Icon'), require('ol/layer/Vector'), require('ol/source/Vector'), require('ol/geom/Point'), require('ol/Feature'), require('ol/proj')) :
  typeof define === 'function' && define.amd ? define(['ol/control/Control', 'ol/style/Style', 'ol/style/Icon', 'ol/layer/Vector', 'ol/source/Vector', 'ol/geom/Point', 'ol/Feature', 'ol/proj'], factory) :
  (global = global || self, global.Geocoder = factory(global.ol.control.Control, global.ol.style.Style, global.ol.style.Icon, global.ol.layer.Vector, global.ol.source.Vector, global.ol.geom.Point, global.ol.Feature, global.ol.proj));
}(this, function (Control, Style, Icon, LayerVector, SourceVector, Point, Feature, proj) { 'use strict';

  Control = Control && Control.hasOwnProperty('default') ? Control['default'] : Control;
  Style = Style && Style.hasOwnProperty('default') ? Style['default'] : Style;
  Icon = Icon && Icon.hasOwnProperty('default') ? Icon['default'] : Icon;
  LayerVector = LayerVector && LayerVector.hasOwnProperty('default') ? LayerVector['default'] : LayerVector;
  SourceVector = SourceVector && SourceVector.hasOwnProperty('default') ? SourceVector['default'] : SourceVector;
  Point = Point && Point.hasOwnProperty('default') ? Point['default'] : Point;
  Feature = Feature && Feature.hasOwnProperty('default') ? Feature['default'] : Feature;
  proj = proj && proj.hasOwnProperty('default') ? proj['default'] : proj;

  var containerId = "gcd-container";
  var buttonControlId = "gcd-button-control";
  var subResetId = "gcd-sub-reset";
  var inputQueryId = "gcd-input-query";
  var inputResetId = "gcd-input-reset";
  var cssClasses = {
  	namespace: "ol-geocoder",
  	spin: "gcd-pseudo-rotate",
  	hidden: "gcd-hidden",
  	address: "gcd-address",
  	country: "gcd-country",
  	city: "gcd-city",
    road: "gcd-road",
    reached: "gcd-reached",
  	olControl: "ol-control",
  	glass: {
  		container: "gcd-gl-container",
  		control: "gcd-gl-control",
      button: "gcd-gl-btn",
      subinput: "gcd-gl-subinput",
      subreset: "gcd-gl-subreset",
  		input: "gcd-gl-input",
  		expanded: "gcd-gl-expanded",
  		reset: "gcd-gl-reset",
  		result: "gcd-gl-result"
  	},
  };
  var vars = {
  	containerId: containerId,
  	buttonControlId: buttonControlId,
  	inputQueryId: inputQueryId,
  	inputResetId: inputResetId,
  	cssClasses: cssClasses
  };

  var _VARS_ = /*#__PURE__*/Object.freeze({
    containerId: containerId,
    buttonControlId: buttonControlId,
    inputQueryId: inputQueryId,
    inputResetId: inputResetId,
    cssClasses: cssClasses,
    'default': vars
  });

  var VARS = _VARS_;

  var EVENT_TYPE = {
    ADDRESSCHOSEN: 'addresschosen',
  };

  var FEATURE_SRC =
    '//cdn.rawgit.com/jonataswalker/map-utils/master/images/marker.png';

  var PROVIDERS = {
    OSM: 'osm',
    VFKRUIAN: 'VfkRuian',
    VFKSKGEO: 'VfkSkGeo',
  };

  var RUIAN = {
    idAdresniMisto: '1',
    idUlice: '4',
    idParcela: '5',
    idKatastralniUzemi: '7',
    idObec: '12',
    idOkres: '15',

    m_urlSuggest: 'https://ags.cuzk.cz/arcgis/rest/services/RUIAN/Vyhledavaci_sluzba_nad_daty_RUIAN/MapServer/exts/GeocodeSOE/suggest',
    m_urlQuery: 'https://ags.cuzk.cz/arcgis/rest/services/RUIAN/Vyhledavaci_sluzba_nad_daty_RUIAN/MapServer/',
    m_msgReached: 'Existuje%20v%C3%ADce%20v%C3%BDsledk%C5%AF%2C%20up%C5%99esn%C4%9Bte%20zad%C3%A1n%C3%AD',

  };

  var DEFAULT_OPTIONS = {
    provider: PROVIDERS.OSM,
    m_phAddress: "Hledat adresu",
    m_phParcel: "Hledat parcelu",
    m_minLenAddress: 4,
    m_minLenParcel: 2,
    m_defLimit: 10,
    m_defLimitRuian: 20,
    m_countrycodes: "CZ",
    m_lang: 'en-US',
    featureStyle: null,
  };

  /**
   * Overwrites obj1's values with obj2's and adds
   * obj2's if non existent in obj1
   * @returns obj3 a new object based on obj1 and obj2
   */
  function mergeOptions(obj1, obj2) {
    var obj3 = {};
    for (var attr1 in obj1) { obj3[attr1] = obj1[attr1]; }
    for (var attr2 in obj2) { obj3[attr2] = obj2[attr2]; }
    return obj3;
  }

  function assert(condition, message) {
    if ( message === void 0 ) message = 'Assertion failed';

    if (!condition) {
      if (typeof Error !== 'undefined') { throw new Error(message); }
      throw message; // Fallback
    }
  }

  function now() {
    // Polyfill for window.performance.now()
    // @license http://opensource.org/licenses/MIT
    // copyright Paul Irish 2015
    // https://gist.github.com/paulirish/5438650
    if ('performance' in window === false) {
      window.performance = {};
    }

    Date.now =
      Date.now ||
      function() {
        // thanks IE8
        return new Date().getTime();
      };

    if ('now' in window.performance === false) {
      var nowOffset = Date.now();

      if (performance.timing && performance.timing.navigationStart) {
        nowOffset = performance.timing.navigationStart;
      }

      window.performance.now = function () { return Date.now() - nowOffset; };
    }

    return window.performance.now();
  }

  function randomId(prefix) {
    var id = now().toString(36);
    return prefix ? prefix + id : id;
  }

  function isNumeric(str) {
    return /^\d+$/.test(str);
  }

  /**
   * @param {Element|Array<Element>} element DOM node or array of nodes.
   * @param {String|Array<String>} classname Class or array of classes.
   * For example: 'class1 class2' or ['class1', 'class2']
   * @param {Number|undefined} timeout Timeout to remove a class.
   */
  function addClass(element, classname, timeout) {
    if (Array.isArray(element)) {
      element.forEach(function (each) { return addClass(each, classname); });
      return;
    }

    var array = Array.isArray(classname) ? classname : classname.split(/\s+/);
    var i = array.length;

    while (i--) {
      if (!hasClass(element, array[i])) {
        _addClass(element, array[i], timeout);
      }
    }
  }

  /**
   * @param {Element|Array<Element>} element DOM node or array of nodes.
   * @param {String|Array<String>} classname Class or array of classes.
   * For example: 'class1 class2' or ['class1', 'class2']
   * @param {Number|undefined} timeout Timeout to add a class.
   */
  function removeClass(element, classname, timeout) {
    if (Array.isArray(element)) {
      element.forEach(function (each) { return removeClass(each, classname, timeout); });
      return;
    }

    var array = Array.isArray(classname) ? classname : classname.split(/\s+/);
    var i = array.length;

    while (i--) {
      if (hasClass(element, array[i])) {
        _removeClass(element, array[i], timeout);
      }
    }
  }

  /**
   * @param {Element} element DOM node.
   * @param {String} classname Classname.
   * @return {Boolean}
   */
  function hasClass(element, c) {
    // use native if available
    return element.classList
      ? element.classList.contains(c)
      : classRegex(c).test(element.className);
  }

  /**
   * Abstraction to querySelectorAll for increased
   * performance and greater usability
   * @param {String} selector
   * @param {Element} context (optional)
   * @param {Boolean} find_all (optional)
   * @return (find_all) {Element} : {Array}
   */
  function find(selector, context, find_all) {
    if ( context === void 0 ) context = window.document;

    var simpleRe = /^(#?[\w-]+|\.[\w-.]+)$/,
      periodRe = /\./g,
      slice = Array.prototype.slice,
      matches = [];

    // Redirect call to the more performant function
    // if it's a simple selector and return an array
    // for easier usage
    if (simpleRe.test(selector)) {
      switch (selector[0]) {
        case '#':
          matches = [$(selector.substr(1))];
          break;
        case '.':
          matches = slice.call(
            context.getElementsByClassName(
              selector.substr(1).replace(periodRe, ' ')
            )
          );
          break;
        default:
          matches = slice.call(context.getElementsByTagName(selector));
      }
    } else {
      // If not a simple selector, query the DOM as usual
      // and return an array for easier usage
      matches = slice.call(context.querySelectorAll(selector));
    }

    return find_all ? matches : matches[0];
  }

  function $(id) {
    id = id[0] === '#' ? id.substr(1, id.length) : id;
    return document.getElementById(id);
  }

  function removeAllChildren(node) {
    while (node.firstChild) { node.removeChild(node.firstChild); }
  }

  function template(html, row) {
    return html.replace(/\{ *([\w_-]+) *\}/g, function (htm, key) {
      var value = row[key] === undefined ? '' : row[key];
      return htmlEscape(value);
    });
  }

  function htmlEscape(str) {
    return String(str)
      .replace(/&/g, '&amp;')
      .replace(/</g, '&lt;')
      .replace(/>/g, '&gt;')
      .replace(/"/g, '&quot;')
      .replace(/'/g, '&#039;');
  }

  function createElement(node, html) {
    var elem;
    if (Array.isArray(node)) {
      elem = document.createElement(node[0]);

      if (node[1].id) { elem.id = node[1].id; }
      if (node[1].classname) { elem.className = node[1].classname; }

      if (node[1].attr) {
        var attr = node[1].attr;
        if (Array.isArray(attr)) {
          var i = -1;
          while (++i < attr.length) {
            elem.setAttribute(attr[i].name, attr[i].value);
          }
        } else {
          elem.setAttribute(attr.name, attr.value);
        }
      }
    } else {
      elem = document.createElement(node);
    }
    elem.innerHTML = html;
    var frag = document.createDocumentFragment();

    while (elem.childNodes[0]) { frag.appendChild(elem.childNodes[0]); }
    elem.appendChild(frag);
    return elem;
  }

  function classRegex(classname) {
    return new RegExp(("(^|\\s+) " + classname + " (\\s+|$)"));
  }

  function _addClass(el, klass, timeout) {
    // use native if available
    if (el.classList) {
      el.classList.add(klass);
    } else {
      el.className = (el.className + ' ' + klass).trim();
    }

    if (timeout && isNumeric(timeout)) {
      window.setTimeout(function () { return _removeClass(el, klass); }, timeout);
    }
  }

  function _removeClass(el, klass, timeout) {
    if (el.classList) {
      el.classList.remove(klass);
    } else {
      el.className = el.className.replace(classRegex(klass), ' ').trim();
    }
    if (timeout && isNumeric(timeout)) {
      window.setTimeout(function () { return _addClass(el, klass); }, timeout);
    }
  }

  var klasses = VARS.cssClasses;

  /**
   * @class Html
   */
  var Html = function Html(base) {
    this.els = this.createControl();
  };

  Html.prototype.createControl = function createControl () {
    var container, containerClass, elements;

    containerClass = (klasses.namespace) + " " + (klasses.glass.container);
    container = createElement(
      ['div', { id: VARS.containerId, classname: containerClass }],
      Html.glass
    );
    elements = {
      container: container,
      control: find('.' + klasses.glass.control, container),
      button: find('.' + klasses.glass.button, container),
      subinput: find('.' + klasses.glass.subinput, container),
      subreset: find('.' + klasses.glass.subreset, container),
      input: find('.' + klasses.glass.input, container),
      reset: find('.' + klasses.glass.reset, container),
      result: find('.' + klasses.glass.result, container),
    };
    return elements;
  };

  /* eslint-disable indent */
  Html.glass = [
    '<div class="',
    klasses.glass.control,
    ' ',
    klasses.olControl,
    '">',
    '<button type="button"',
    ' id="',
    VARS.buttonControlId,
    '"',
    ' class="',
    klasses.glass.button,
    '"></button>',
    '<span id="gcd-input-subquery" class="gcd-gl-subinput">Katastrln zem</span>',
//    '<input type="text" id="gcd-input-subquery" class="gcd-gl-subinput" autocomplete="off" placeholder="Katastrln zem">',
    '<a id="gcd-input-subreset" class="gcd-gl-subreset gcd-hidden"></a>',
    //'<div class="gcd-gl-sub">',
    //'<span class="label label-default">Default</span>',
    //'<a id="gcd-sub-reset" class="gcd-gl-subreset"></a>',
    //'</div>',
    '<input type="text"',
    ' id="',
    VARS.inputQueryId,
    '"',
    ' class="',
    klasses.glass.input,
    '"',
    ' autocomplete="off" placeholder="Search ...">',
    '<a',
    ' id="',
    VARS.inputResetId,
    '"',
    ' class="',
    klasses.glass.reset,
    ' ',
    klasses.hidden,
    '"',
    '></a>',
    '</div>',
    '<ul class="',
    klasses.glass.result,
    '"></ul>' ].join('');
    /* eslint-enable indent */

  /**
   * @class OpenStreet
   */
  var OpenStreet = function OpenStreet(base) {
    this.m_url = 'https://nominatim.openstreetmap.org/search/';
    this.m_addressdetails = 1;
    this.m_Base = base;
  };

  OpenStreet.prototype.getParameters = function getParameters (opt) {
    return {
      url: this.m_url,
      params: {
        q: opt.query,
        format: 'json',
        addressdetails: this.m_addressdetails,
        limit: this.m_Base.options.m_defLimit,
        countrycodes: this.m_Base.options.m_countrycodes,
        'accept-language': this.m_Base.options.m_lang,
      },
    };
  };

  OpenStreet.prototype.handleResponse = function handleResponse (par, results) {
    if (!results.length) { return; }
    return results.map(function (it) {
      var retVal = {
        lon: it.lon,
        lat: it.lat,
        bbox: null,
        address: {
          name: it.display_name,
          road: it.address.road || '',
          houseNumber: it.address.house_number || '',
          postcode: it.address.postcode,
          city: it.address.city || it.address.town,
          state: it.address.state,
          country: it.address.country,
        },
        original : it.address,
      };

      if (it.boundingbox && 3 < it.boundingbox.length) {
        retVal.bbox = [parseFloat(it.boundingbox[2]), parseFloat(it.boundingbox[0]), parseFloat(it.boundingbox[3]), parseFloat(it.boundingbox[1])];
      }
      return retVal;
    });
  };

  OpenStreet.prototype.onChosen = function onChosen(place, addressHtml) {
    var coord_ = [parseFloat(place.lon), parseFloat(place.lat)];
    var projection = this.m_Base.getMap().getView().getProjection();
    var coord = proj.transform(coord_, 'EPSG:4326', projection);
    var bbox = place.bbox;

    if (bbox)
      bbox = proj.transformExtent(bbox, 'EPSG:4326', projection);

    VfkDispatchAddressChosen(this.m_Base, coord, bbox);
  }

  OpenStreet.prototype.clearSub = function clearSub() {
  }

  // features[idx].geometry.rings features[idx].geometry.paths
  function VfkBBoxFromCoordsList(bbox, coordsList) {
    var minX = null, maxX = null, minY = null, maxY = null;
    if (bbox) {
      minX = bbox[0];
      maxX = bbox[2];
      minY = bbox[1];
      maxY = bbox[3];
    }

    for (var coords of coordsList) {

      for (var point of coords) {

        if (minX === null || point[0] < minX)
          minX = point[0];
        if (maxX === null || point[0] > maxX)
          maxX = point[0];

        if (minY === null || point[1] < minY)
          minY = point[1];
        if (maxY === null || point[1] > maxY)
          maxY = point[1];
      }
    }
    if (minX !== null && maxX !== null && minY !== null && maxY !== null)
      bbox = [minX, minY, maxX, maxY];
    return bbox;
  }

  function VfkBBoxFromFeatures(features) {
    var bbox = null;
    for (var feat of features) {
      if (!("geometry" in feat)) continue;

      var coordsList = null;
      if ("rings" in feat.geometry)
        coordsList = feat.geometry.rings;
      else if ("paths" in feat.geometry)
        coordsList = feat.geometry.paths;

      if (coordsList !== null)
        bbox = VfkBBoxFromCoordsList(bbox, coordsList);
    }
    return bbox;
  }

  // "Obec", "Ulice", "AdresniMisto"
  function VfkRuianAddress(base, layId, res) {
    var bbox = null, aCenter = null;
    if ("extent" in res)
      bbox = [res.extent.xmin, res.extent.ymin, res.extent.xmax, res.extent.ymax];
    else if (("features" in res) && res.features.length) {
      bbox = VfkBBoxFromFeatures(res.features);
      if ("geometry" in res.features[0] && "x" in res.features[0].geometry && "y" in res.features[0].geometry)
        aCenter = [res.features[0].geometry.x, res.features[0].geometry.y];
    }
    base.m_vfkNominatim.clearResults();
    base.m_vfkNominatim.provider.clearSub();

    VfkDispatchAddressChosen(base, aCenter, bbox);
  }

  function VfkRuianKatastralniUzemi(base, layId, res) {
    var bbox = null;
    if ("extent" in res)
      bbox = [res.extent.xmin, res.extent.ymin, res.extent.xmax, res.extent.ymax];
    else if (("features" in res) && res.features.length)
      bbox = VfkBBoxFromFeatures(res.features);

    base.m_vfkNominatim.clearResults();
    if (("features" in res) && res.features.length && ("attributes" in res.features[0])) {
      base.m_vfkNominatim.expandSub(res.features[0].attributes.nazev);
      base.m_vfkNominatim.provider.m_subTable = RUIAN.idParcela; // Parcela
      base.m_vfkNominatim.provider.m_subData = res.features[0].attributes;
    }
    VfkDispatchAddressChosen(base, null, bbox);
  }

  function VfkRuainResolveObec(element, feat) {

    if (!element.firstChild)
      return;
    if (!element.firstChild.lastChild)
      return;
    var elSpan = element.firstChild.lastChild;
    var url = RUIAN.m_urlQuery + RUIAN.idOkres + "/query";
    var params = {
      where: "kod = " + feat.attributes.okres,
      units: "esriSRUnit_Meter",
      outFields: "objectid,kod,nazev",
      returnGeometry: "true",
      f: 'json',
    };
    url = VfkEncodeUrlXhr(url, params);
    var config = {
      method: 'GET',
      mode: 'cors',
      credentials: 'same-origin',
    };

    fetch(url, config)
      .then(VfkParseJson)
      .then(function (res) {
        if (("features" in res) && res.features.length && ("attributes" in res.features[0])) {
          elSpan.innerText = "Obec, Okr. " + res.features[0].attributes.nazev;
        }
      })
      .catch(function (err) {
      });
  }

  function VfkSkGeoResolveParcel(base, res) {
    if (!("features" in res)) return null;

    var bbox = null;
    for (var feature of res.features) {
      if (!("attributes" in feature)) continue;
      if (!("PARCEL_NUMBER" in feature.attributes)) continue;
      if (!("geometry" in feature) || !("rings" in feature.geometry)) continue;

      bbox = VfkBBoxFromCoordsList(bbox, feature.geometry.rings);
      if (bbox !== null)
        bbox = proj.transformExtent(bbox, 'EPSG:4326', base.getMap().getView().getProjection());
    }
    VfkDispatchAddressChosen(base, null, bbox);
    return null;
  }

  function VfkDispatchAddressChosen(base, coord, bbox) {
    base.dispatchEvent({
      type: EVENT_TYPE.ADDRESSCHOSEN,
      coordinate: coord,
      bbox: bbox,
    });
  }

  /**
 * @class VfkRuian
 */
  var VfkRuian = function VfkRuian(base) {
    this.m_urlQuery = 'https://ags.cuzk.cz/arcgis/rest/services/RUIAN/Vyhledavaci_sluzba_nad_daty_RUIAN/MapServer/';
    this.m_Base = base;
  };

  VfkRuian.prototype.getParameters = function getParameters(opt) {
    var retPars = null;
    if ("m_subData" in this && this.m_subData) {
      retPars = new Array({
        m_id: RUIAN.idParcela,
        url: RUIAN.m_urlQuery + this.m_subTable + "/query",
        params: {
          where: "katastralniuzemi = " + this.m_subData.kod + " AND cisloparcely LIKE \'%" + opt.query + "%\'",
          units: "esriSRUnit_Meter",
          outFields: "objectid,id,kmenovecislo,poddelenicisla,druhcislovanikod,katastralniuzemi,cisloparcely",
          resultRecordCount: this.m_Base.options.m_defLimit,
          f: 'json',
        },
      });
    } else {
//  const classes = `header ${isLargeScreen() ? "" : item.isCollapsed ? "icon-expander" : "icon-collapser"}`;
//  const classes = `header ${isLargeScreen() ? "" : `icon-${item.isCollapsed ? "expander" : "collapser"}`}`;
      var tmpQ = opt.query.toLowerCase();
      tmpQ = tmpQ.replaceAll(',', '');
      tmpQ = tmpQ.replaceAll(' ', '%');
      var tmpPars = [
        {
          retId: RUIAN.idAdresniMisto,
          retUrl: RUIAN.m_urlQuery + RUIAN.idAdresniMisto + "/query",
          retWhere: "LOWER(adresa) LIKE \'%" + tmpQ + "%\'",
          retOut: "objectid,kod,cislodomovni,cisloorientacni,cisloorientacnipismeno,psc,ulice,adresa",
          retCount: this.m_Base.options.m_defLimit,
        },
        {
          retId: RUIAN.idUlice,
          retUrl: RUIAN.m_urlQuery + RUIAN.idUlice + "/query",
          retWhere: "LOWER(ulice) LIKE \'%" + tmpQ + "%\'",
          retOut: "objectid,kod,nazev,obec,ulice",
          retCount: this.m_Base.options.m_defLimitRuian,
        },
        {
          retId: RUIAN.idKatastralniUzemi,
          retUrl: RUIAN.m_urlQuery + RUIAN.idKatastralniUzemi + "/query",
          retWhere : "LOWER(nazev) LIKE \'%" + tmpQ + "%\'",
          retOut: "objectid,kod,nazev,obec",
          retCount: this.m_Base.options.m_defLimit,
        },
        {
          retId: RUIAN.idObec,
          retUrl: RUIAN.m_urlQuery + RUIAN.idObec + "/query",
          retWhere : "LOWER(nazev) LIKE \'%" + tmpQ + "%\'",
          retOut: "objectid,kod,nazev,okres",
          retCount: this.m_Base.options.m_defLimit,
        }
      ]

      for (var tmpPar of tmpPars) {
        var retPar = {
          m_id: tmpPar.retId,
          m_count: tmpPar.retCount,
          url: tmpPar.retUrl,
          params: {
            where: tmpPar.retWhere,
            units: "esriSRUnit_Meter",
            outFields: tmpPar.retOut,
            returnGeometry: "true",
            returnTrueCurves: "false",
            resultRecordCount: tmpPar.retCount,
            returnExtentOnly: "false",
            featureEncoding: "esriDefault",
            f: 'json',
          },
        };
        if (retPars)
          retPars.push(retPar);
        else
          retPars = new Array(retPar);
      }
    }
    return retPars;
  //  if ("m_subData" in this && this.m_subData) {
  //    var retUrl = "https://ags.cuzk.cz/arcgis/rest/services/RUIAN/Vyhledavaci_sluzba_nad_daty_RUIAN/MapServer/";
  //    retUrl += this.m_subTable;
  //    retUrl += "/query";
  //    var retWhere = "katastralniuzemi = ";
  //    retWhere += this.m_subData.kod;
  //    retWhere += " AND cisloparcely LIKE \'%";
  //    retWhere += opt.query;
  //    retWhere += "%\'";
  //    return {
  //      url : retUrl,
  //      params: {
  //        where: retWhere,
  //        units: "esriSRUnit_Meter",
  //        outFields: "objectid,id,kmenovecislo,poddelenicisla,druhcislovanikod,katastralniuzemi,cisloparcely",
  //        resultRecordCount: this.m_Base.options.m_defLimit,
  //        f: 'json',
  //      },
  //    };
  //  } else {
  //    return {
  //      url: RUIAN.m_urlSuggest,
  //      params: {
  //        text: opt.query,
  //        f: 'json',
  //        maxSuggestions: this.m_Base.options.m_defLimitRuian,
  //      },
  //    };
  //  }
  };

  VfkRuian.prototype.handleResponse = function handleResponse(par, results) {
    var retVals = null, retVal = 0;

    if ("features" in results) {
      for (var feat of results.features) {
        var retName = null, retCat = null, retFn = null;
        if (par.m_id === RUIAN.idAdresniMisto) {
          retName = feat.attributes.adresa;
          retCat = "Adresa";
        } else if (par.m_id === RUIAN.idUlice) {
          retName = feat.attributes.ulice;
          retCat = "Ulice";
        } else if (par.m_id === RUIAN.idParcela) {
          retName = feat.attributes.cisloparcely;
          retCat = feat.attributes.druhcislovanikod == 1 ? "Stavebni" : "Pozemkova";
        } else if (par.m_id === RUIAN.idKatastralniUzemi) {
          retName = feat.attributes.nazev + " ( " + feat.attributes.kod + " )";
          retCat = "KatastralniUzemi";
        } else if (par.m_id === RUIAN.idObec) {
          retName = feat.attributes.nazev;
          retCat = "Obec";
          retFn = VfkRuainResolveObec;
        } else
          continue;

        retVal = {
          address: {
            name: retName,
            category: retCat,
          },
          m_id: par.m_id,
          m_fnResolve : retFn,
          original: feat,
        };
        if (retVals)
          retVals.push(retVal);
        else
          retVals = new Array(retVal);
      }
    }

    //if ("suggestions" in results && results.suggestions.length) {
    //  for (var suggest of results.suggestions) {

    //    if ((suggest.type != "KatastralniUzemi") && (suggest.type != "Obec") && (suggest.type != "Ulice") && (suggest.type != "AdresniMisto")) continue;

    //    retVal = {
    //      address: {
    //        name: suggest.text,
    //        category: suggest.type,
    //      },
    //      original: suggest,
    //    };
    //    if (retVals)
    //      retVals.push(retVal);
    //    else
    //      retVals = new Array(retVal);
    //  }
    //} else if ("features" in results && results.features.length) {
    //  for (var feat of results.features) {
    //    retVal = {
    //      address: {
    //        name: feat.attributes.cisloparcely,
    //        category: feat.attributes.druhcislovanikod == 1 ? "Stavebni" : "Pozemkova",
    //      },
    //      original: feat,
    //    };
    //    if (retVals)
    //      retVals.push(retVal);
    //    else
    //      retVals = new Array(retVal);
    //  }
    //}
    return retVals;
  };

    VfkRuian.prototype.onChosen = function onChosen(place, addressHtml) {
    if ("magicKey" in place.original) {
      var keys = place.original.magicKey.split('_');
      if (!keys.length || keys.length < 2) return;
      if (!this.m_Base.m_vfkRuianMap.has(keys[0])) return;
      if (!this.m_Base.m_vfkRuianMap.get(keys[0]).m_fnChosen) return;

      var Base$1 = this.m_Base;
      var url = "https://ags.cuzk.cz/arcgis/rest/services/RUIAN/Vyhledavaci_sluzba_nad_daty_RUIAN/MapServer/" + keys[0] + "/query";

      var pars = {
        objectIds: keys[1],
        geometryType: "esriGeometryEnvelope",
        spatialRel: "esriSpatialRelIntersects",
        units: "esriSRUnit_Meter",
        outFields: this.m_Base.m_vfkRuianMap.get(keys[0]).m_outFields,
        returnGeometry: "true",
        returnTrueCurves: "false",
        returnIdsOnly: "false",
        returnCountOnly: "false",
        returnZ: "false",
        returnM: "false",
        returnDistinctValues: "false",
        returnExtentOnly: "false",
        sqlFormat: "none",
        featureEncoding: "esriDefault",
        f: "json",
      };
      url = VfkEncodeUrlXhr(url, pars);
      var config = {
        method: 'GET',
        mode: 'cors',
        credentials: 'same-origin',
      };
      fetch(url, config)
        .then(VfkParseJson)
        .then(function (res) {
          Base$1.m_vfkRuianMap.get(keys[0]).m_fnChosen(Base$1, keys[0], res);
        })
        .catch(VfkTestReject);
    } else if ("attributes" in place.original && "geometry" in place.original) {
      this.m_Base.m_vfkNominatim.clearResults();

      var bbox = null, aCenter = null;
      if ("rings" in place.original.geometry)
        bbox = VfkBBoxFromCoordsList(bbox, place.original.geometry.rings);
      else if ("paths" in place.original.geometry)
        bbox = VfkBBoxFromCoordsList(bbox, place.original.geometry.paths);
      else if ("x" in place.original.geometry && "y" in place.original.geometry)
        aCenter = [place.original.geometry.x, place.original.geometry.y];

      if (place.m_id === RUIAN.idKatastralniUzemi) {
        this.m_Base.m_vfkNominatim.expandSub(place.original.attributes.nazev);
        this.m_subTable = RUIAN.idParcela; // Parcela
        this.m_subData = place.original.attributes;
      }

      VfkDispatchAddressChosen(this.m_Base, aCenter, bbox);
    }
  };

  VfkRuian.prototype.clearSub = function clearSub() {
    this.m_subTable = null;
    this.m_subData = null;
  }

    /**
   * @class VfkSkGeo
   */
  var VfkSkGeo = function VfkSkGeo(base) {
    this.m_url = 'https://zbgis.skgeodesy.sk/mkzbgis/api/suggest/kataster';
    this.m_Base = base;
    this.m_subData = null;
  };

  VfkSkGeo.prototype.getParameters = function getParameters(opt) {
    var retUrl = this.m_url;
    if (this.m_subData)
      retUrl += "/" + this.m_subData.municipalityCode + "/cu/" + this.m_subData.id;
    return new Array({
      url: retUrl,
      params: {
        q: opt.query,
      },
    });
  };

  VfkSkGeo.prototype.handleResponse = function handleResponse(par, results) {
    if (!results.items.length) { return; }

    var retVals = null;
    for (var it of results.items) {
      var doIt = false;
      if (("category" in it.data) && ((it.data.category === "parcela-c") || (it.data.category === "parcela-e")))
        doIt = true;
      else if (("extent" in it.data) && ("coordinates" in it.data.extent)) {
        var coords = it.data.extent.coordinates;
        if (1 < coords.length && coords[0][0] !== 0 && coords[0][1] !== 0 && coords[1][0] !== 0 && coords[1][1] !== 0)
          doIt = true;
      }
      if (!doIt)
        continue;
      var retVal = {
        address: {
          name: it.data.text,
          category: it.data.category,
        },
        original: it.data,
      };
      if (retVals)
        retVals.push(retVal);
      else
        retVals = new Array(retVal);
    }
    return retVals;
  };

  VfkSkGeo.prototype.onChosen = function onChosen(place, addressHtml) {
    var Base$1 = this.m_Base;
    var extent = null;

    if (("category" in place.original) && place.original.category.includes("katastr")) {
      this.m_subData = place.original;
      this.m_Base.m_vfkNominatim.clearResults();
      this.m_Base.m_vfkNominatim.expandSub(place.original.text);
      extent = place.original.extent;
    }
    else if (("category" in place.original) && ((place.original.category === "parcela-c") || (place.original.category === "parcela-e"))) {
      var url = null;
      this.m_Base.m_vfkNominatim.clearResults();
      if (place.original.category === "parcela-c")
        url = 'https://kataster.skgeodesy.sk/eskn/rest/services/VRM/parcels_c_view/MapServer/0/query?objectIds=';
      else if (place.original.category === "parcela-e")
        url = 'https://kataster.skgeodesy.sk/eskn/rest/services/VRM/parcels_e_view/MapServer/0/query?objectIds=';
      url += place.original.id.toString();
      url += '&returnGeometry=true&outSR=4326&f=json';
      var config = {
        method: 'GET',
        mode: 'cors',
        credentials: 'same-origin',
      };
      fetch(url, config)
        .then(VfkParseJson)
        .then(function (res) {
          VfkSkGeoResolveParcel(Base$1, res);
        })
        .catch(VfkTestReject);
    } else {// place.original.category === "adresa"
      this.m_Base.m_vfkNominatim.clearResults();
      extent = place.original.extent;
//      this.m_Base.m_vfkNominatim.collapseClearSub(); // left subdata expanded
    }

    if (extent) {
      var coords = extent.coordinates, bbox = null;
      var midX = 0.5 * (Number(coords[0][0]) + Number(coords[1][0]));
      var midY = 0.5 * (Number(coords[0][1]) + Number(coords[1][1]));
      if ((coords[0][0] !== coords[1][0]) && (coords[0][1] !== coords[1][1])) {
        bbox = [parseFloat(coords[0][0]), parseFloat(coords[0][1]), parseFloat(coords[1][0]), parseFloat(coords[1][1])];
      } else {
        bbox = [midX - 0.0001, midY - 0.0001, midX + 0.0001, midY + 0.0001];
      }
      var projection = this.m_Base.getMap().getView().getProjection();
      bbox = proj.transformExtent(bbox, 'EPSG:4326', projection);

      VfkDispatchAddressChosen(this.m_Base, null, bbox);
    }
  }

  VfkSkGeo.prototype.clearSub = function clearSub() {
    this.m_subData = null;
  }

  var klasses$1 = VARS.cssClasses;

  /**
   * @class Nominatim
   */
  var Nominatim = function Nominatim(base, els) {
    this.m_Base = base;
    this.m_minLenCur = base.options.m_minLenAddress;
    this.m_lastQuery = '';

    this.layerName = randomId('geocoder-layer-');
    this.layer = new LayerVector({
      name: this.layerName,
      source: new SourceVector(),
    });

    this.provider = this.newProvider();

    this.els = els;
    this.registeredListeners = { mapClick: false };
    this.setListeners();
  };

  Nominatim.prototype.setListeners = function setListeners () {
    var this$1 = this, timeout;

    var openSearch = function () {
      if (hasClass(this$1.els.control, klasses$1.glass.expanded)) {
        this$1.collapseInput();
      } else {
        this$1.expandInput();
      }
    };

    var onSubresetClick = function () {
      this$1.initInput();
    };

    var onInputKeyPress = function (evt) {
      var value = evt.target.value.trim();
      var hit = evt.key
        ? evt.key === 'Enter'
        : evt.which
        ? evt.which === 13
        : evt.keyCode
        ? evt.keyCode === 13
        : false;

      if (hit) {
        evt.preventDefault();
        if (value.length)
          this$1.queryProvider(value);
      }
    };
    var onResetClick = function (evt) {
      this$1.clearResults();
      this$1.els.input.focus();
    };
    var onInputInput = function (evt) {
      var value = evt.target.value.trim();

      value.length ? removeClass(this$1.els.reset, klasses$1.hidden) : addClass(this$1.els.reset, klasses$1.hidden);

      if (value.length >= this$1.m_minLenCur) {
        timeout && clearTimeout(timeout);
        timeout = setTimeout(function () {
          this$1.queryProvider(value);
        }, 200);
      }
    };
    this.els.button.addEventListener('click', openSearch, false);
    this.els.subreset.addEventListener('click', onSubresetClick, false);
    this.els.input.addEventListener('keypress', onInputKeyPress, false);
    this.els.input.addEventListener('input', onInputInput, false);
    this.els.reset.addEventListener('click', onResetClick, false);
  };

  Nominatim.prototype.queryProvider = function queryProvider(q) {
    if (q === this.m_lastQuery)
      return;
    this.m_lastQuery = q;
    if (!this.provider)
      this.provider = this.newProvider();

    var parArr = this.provider.getParameters({
      query: q,
    });
    var parLen = parArr.length, countReached = false;
    removeAllChildren(this.els.result);
    addClass(this.els.reset, klasses$1.spin);

    var this$1 = this;
    parArr.forEach(function (par) {
      var url = VfkEncodeUrlXhr(par.url, par.params);
      var config = {
        method: 'GET',
        mode: 'cors',
        credentials: 'same-origin',
      };

      fetch(url, config)
        .then(VfkParseJson)
        .then(function (res) {
          parLen--;
          if (this$1.m_lastQuery === q) {
            var res_ = this$1.provider.handleResponse(par, res);
            if (res_) {
              this$1.createList(res_);
              this$1.listenMapClick();
              if ("m_count" in par && par.m_count <= res_.length)
                countReached = true;
            }

            if (parLen <= 0) {
              removeClass(this$1.els.reset, klasses$1.spin);
              if (countReached) {
                var html = "<span class=\"" + (klasses$1.reached) + "\">" + decodeURIComponent(RUIAN.m_msgReached) + "</span>";
                var li = createElement('li', html);
                this$1.els.result.appendChild(li);
              }
            }
          }
        })
        .catch(function (err) {
          parLen--;
          if (parLen <= 0)
            removeClass(this$1.els.reset, klasses$1.spin);

          var li = createElement(
            'li',
            '<h5>Error! No internet connection?</h5>'
          );
          this$1.els.result.appendChild(li);
        });
    });
  };

  Nominatim.prototype.createList = function createList (response) {
      var this$1 = this;

    var ul = this.els.result;

    response.forEach(function (row) {
      var addressHtml;

      switch (this$1.m_Base.options.provider) {
        case PROVIDERS.OSM:
          addressHtml = "<span class=\"" + (klasses$1.road) + "\">" + (row.address.name) + "</span>";
          break;
        default:
          addressHtml = this$1.addressTemplate(row.address);
      }

      var html = "<a href=\"#\">" + addressHtml + "</a>";
      var li = createElement('li', html);

      li.addEventListener(
        'click',
        function (evt) {
          evt.preventDefault();
          this$1.provider.onChosen(row, addressHtml);
        },
        false
      );
      if ("m_fnResolve" in row && row.m_fnResolve)
        row.m_fnResolve(li, row.original);

      ul.appendChild(li);
    });
  };

  Nominatim.prototype.createFeature = function createFeature (coord) {
    var feature = new Feature(new Point(coord));
    this.addLayer();
    feature.setStyle(this.options.featureStyle);
    feature.setId(randomId('geocoder-ft-'));
    this.getSource().addFeature(feature);
    return feature;
  };

  Nominatim.prototype.addressTemplate = function addressTemplate (address) {
    var html = [];
    if (address.name) {
      html.push(['<span class="', klasses$1.road, '">{name}</span>'].join(''));
    }
    if (address.road || address.building || address.house_number) {
      html.push(
        [
          '<span class="',
          klasses$1.road,
          '">{building} {road} {house_number}</span>' ].join('')
      );
    }
    if (address.city || address.town || address.village) {
      html.push(
        [
          '<span class="',
          klasses$1.city,
          '">{postcode} {city} {town} {village}</span>' ].join('')
      );
    }
    if (address.state || address.country) {
      html.push(
        ['<span class="', klasses$1.country, '">{state} {country}</span>'].join(
          ''
        )
      );
    }
    if (address.category) {
      html.push(
        ['<span class="', klasses$1.country, '">{category}</span>'].join(
          ''
        )
      );
    }
    return template(html.join('<br>'), address);
  };

  Nominatim.prototype.newProvider = function newProvider () {
    /*eslint default-case: 0*/
    switch (this.m_Base.options.provider) {
      case PROVIDERS.OSM:
        return new OpenStreet(this.m_Base);
      case PROVIDERS.VFKRUIAN:
        return new VfkRuian(this.m_Base);
      case PROVIDERS.VFKSKGEO:
        return new VfkSkGeo(this.m_Base);
      default:
        return this.options.provider;
    }
  };

  Nominatim.prototype.initInput = function initInput() {
    var this$1 = this;
    this.m_minLenCur = this.m_Base.options.m_minLenAddress;
    this.els.input.placeholder = this.m_Base.options.m_phAddress;
    this.clearResults();
    this.collapseClearSub();
    window.setTimeout(function () { return this$1.els.input.focus(); }, 100);
  };

  Nominatim.prototype.expandInput = function expandInput() {
    addClass(this.els.control, klasses$1.glass.expanded);
    this.initInput();
    this.listenMapClick();
  };

  Nominatim.prototype.collapseInput = function collapseInput () {
    this.els.input.blur();
    this.collapseClearSub();
    this.clearResults();
    addClass(this.els.reset, klasses$1.hidden);
    removeClass(this.els.control, klasses$1.glass.expanded);
  };

  Nominatim.prototype.expandSub = function expandSub(subTxt) {
    this.m_minLenCur = this.m_Base.options.m_minLenParcel;
    this.els.subinput.innerHTML = subTxt;
    this.els.input.placeholder = this.m_Base.options.m_phParcel;
    removeClass(this.els.subinput, klasses$1.hidden);
    removeClass(this.els.subreset, klasses$1.hidden);
  };

  Nominatim.prototype.collapseClearSub = function collapseClearSub() {
    addClass(this.els.subinput, klasses$1.hidden);
    addClass(this.els.subreset, klasses$1.hidden);
    if (this.provider)
      this.provider.clearSub();
  };

  Nominatim.prototype.listenMapClick = function listenMapClick () {
    // already registered
    if (this.registeredListeners.mapClick) { return; }

    var this$1 = this;
    var mapElement = this.m_Base.getMap().getTargetElement();
    this.registeredListeners.mapClick = true;

    //one-time fire click
    mapElement.addEventListener(
      'click',
      {
        handleEvent: function(evt) {
          this$1.clearResults();
          mapElement.removeEventListener(evt.type, this, false);
          this$1.registeredListeners.mapClick = false;
        },
      },
      false
    );
  };

  Nominatim.prototype.clearResults = function clearResults() {
    this.m_lastQuery = '';
    this.els.input.value = '';
    removeClass(this.els.input, klasses$1.spin);
    if (!hasClass(this.els.reset, klasses$1.hidden))
      addClass(this.els.reset, klasses$1.hidden);
    removeAllChildren(this.els.result);
  };

  Nominatim.prototype.getSource = function getSource () {
    return this.layer.getSource();
  };

  Nominatim.prototype.addLayer = function addLayer () {
      var this$1 = this;

    var found = false;
    var map = this.m_Base.getMap();

    map.getLayers().forEach(function (layer) {
      if (layer === this$1.layer) { found = true; }
    });
    if (!found) { map.addLayer(this.layer); }
  };

  /**
   * @class Base
   * @extends {ol.control.Control}
   */
  var Base = /*@__PURE__*/(function (Control) {
    function Base(options) {
      if ( options === void 0 ) options = {};

      if (!(this instanceof Base)) { return new Base(); }

      DEFAULT_OPTIONS.featureStyle = [
        new Style({ image: new Icon({ scale: 0.7, src: FEATURE_SRC }) }) ];

      this.options = mergeOptions(DEFAULT_OPTIONS, options);

      this.m_vfkNominatim = null;
      this.m_vfkRuianMap = null;
      this.m_vfkRuianMap = new Map();
      this.m_vfkRuianMap.set("0", {
        m_layName: "ParcelaDefinicniBod",
        m_fnChosen: null,
        m_outFields: null,
      });
      this.m_vfkRuianMap.set("1", {
        m_layName: "AdresniMisto",
        m_fnChosen: VfkRuianAddress,
        m_outFields: "kod,cislodomovni,cisloorientacni,cisloorientacnipismeno,psc,stavebniobjekt,ulice,adresa",
      });
      this.m_vfkRuianMap.set("4", {

        m_layName: "Ulice",
        m_fnChosen: VfkRuianAddress,
        m_outFields: "kod,nazev,obec,ulice",
      });
      this.m_vfkRuianMap.set("6", {
        m_layName: "ZakladniSidelniJednotka",
        m_fnChosen: null,
        m_outFields: null,
      });
      this.m_vfkRuianMap.set("7", {
        m_layName: "KatastralniUzemi",
        m_fnChosen: VfkRuianKatastralniUzemi,
        m_outFields: "kod,nazev,obec",
      });
      this.m_vfkRuianMap.set("8", {
        m_layName: "MestskyObvodMestskaCastVeStatutarnimMesteAHlavnimMestePraze",
        m_fnChosen: null,
        m_outFields: null,
      });
      this.m_vfkRuianMap.set("10", {
        m_layName: "SpravniObvodVHlavnimMestePraze",
        m_fnChosen: null,
        m_outFields: null,
      });
      this.m_vfkRuianMap.set("11", {
        m_layName: "CastObce",
        m_fnChosen: null,
        m_outFields: null,
      });
      this.m_vfkRuianMap.set("12", {
        m_layName: "Obec",
        m_fnChosen: VfkRuianAddress,
        m_outFields: "kod,statuskod,nazev,okres",
      });
      this.m_vfkRuianMap.set("13", {
        m_layName: "ObecSPoverenymObecnimUradem",
        m_fnChosen: null,
        m_outFields: null,
      });
      this.m_vfkRuianMap.set("14", {
        m_layName: "ObecSRozsirenouPusobnosti",
        m_fnChosen: null,
        m_outFields: null,
      });
      this.m_vfkRuianMap.set("15", {
        m_layName: "Okres",
        m_fnChosen: null,
        m_outFields: null,
      });
      this.m_vfkRuianMap.set("17", {
        m_layName: "VyssiUzemneSamospravnyCelek",
        m_fnChosen: null,
        m_outFields: null,
      });

      var $html = new Html(this);

      this.m_vfkNominatim = new Nominatim(this, $html.els);

      Control.call(this, { element: this.m_vfkNominatim.els.container });
    }

    if ( Control ) Base.__proto__ = Control;
    Base.prototype = Object.create( Control && Control.prototype );
    Base.prototype.constructor = Base;

    /**
     * @return {ol.layer.Vector} Returns the layer created by this control
     */
    Base.prototype.getLayer = function getLayer() {
      return this.m_vfkNominatim.layer;
    };

    /**
     * @return {ol.source.Vector} Returns the source created by this control
     */
    Base.prototype.getSource = function getSource () {
      return this.getLayer().getSource();
    };

    /**
     * Set a new provider
     * @param {String} provider
     */
    Base.prototype.setProvider = function setProvider (provider) {
      this.options.provider = provider;
    };

    return Base;
  }(Control));

  return Base;

}));
